/// Database access demonstration:
/// Uses OleDB to query Access, Excel, and Text file databases.
/// Extracts schema information to get list of table names in database.
/// Displays data for chosen table, using DataGrid control as well as manual processing.
/// -Chris North, Virginia Tech


using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;

namespace DatabaseDemo
{
	/// <summary>
	/// Summary description for Form1.
	/// </summary>
	public class Form1 : System.Windows.Forms.Form
	{
		private System.Data.OleDb.OleDbConnection oleDbConnection1;
		private System.Data.OleDb.OleDbCommand oleDbCommand1;
		private System.Data.OleDb.OleDbDataAdapter oleDbDataAdapter1;
		private System.Data.DataSet dataSet1;

		private System.Windows.Forms.TextBox textBox1;
		private System.Windows.Forms.OpenFileDialog openFileDialog1;
		private System.Windows.Forms.Button button2;
		private System.Windows.Forms.DataGrid dataGrid1;
		private System.Windows.Forms.ListBox listBox1;
		private System.Windows.Forms.Button button3;

		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public Form1()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			//
			// TODO: Add any constructor code after InitializeComponent call
			//
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.textBox1 = new System.Windows.Forms.TextBox();
			this.openFileDialog1 = new System.Windows.Forms.OpenFileDialog();
			this.button2 = new System.Windows.Forms.Button();
			this.dataGrid1 = new System.Windows.Forms.DataGrid();
			this.listBox1 = new System.Windows.Forms.ListBox();
			this.button3 = new System.Windows.Forms.Button();
			((System.ComponentModel.ISupportInitialize)(this.dataGrid1)).BeginInit();
			this.SuspendLayout();
			// 
			// textBox1
			// 
			this.textBox1.Location = new System.Drawing.Point(152, 272);
			this.textBox1.Multiline = true;
			this.textBox1.Name = "textBox1";
			this.textBox1.ScrollBars = System.Windows.Forms.ScrollBars.Vertical;
			this.textBox1.Size = new System.Drawing.Size(584, 264);
			this.textBox1.TabIndex = 0;
			this.textBox1.Text = "textBox1";
			// 
			// button2
			// 
			this.button2.Location = new System.Drawing.Point(16, 8);
			this.button2.Name = "button2";
			this.button2.Size = new System.Drawing.Size(120, 24);
			this.button2.TabIndex = 2;
			this.button2.Text = "Open Database ...";
			this.button2.Click += new System.EventHandler(this.button2_Click);
			// 
			// dataGrid1
			// 
			this.dataGrid1.DataMember = "";
			this.dataGrid1.HeaderForeColor = System.Drawing.SystemColors.ControlText;
			this.dataGrid1.Location = new System.Drawing.Point(152, 8);
			this.dataGrid1.Name = "dataGrid1";
			this.dataGrid1.Size = new System.Drawing.Size(584, 256);
			this.dataGrid1.TabIndex = 3;
			// 
			// listBox1
			// 
			this.listBox1.Location = new System.Drawing.Point(16, 48);
			this.listBox1.Name = "listBox1";
			this.listBox1.Size = new System.Drawing.Size(120, 173);
			this.listBox1.TabIndex = 4;
			// 
			// button3
			// 
			this.button3.Location = new System.Drawing.Point(16, 240);
			this.button3.Name = "button3";
			this.button3.Size = new System.Drawing.Size(120, 24);
			this.button3.TabIndex = 5;
			this.button3.Text = "Show table >>";
			this.button3.Click += new System.EventHandler(this.button3_Click);
			// 
			// Form1
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(752, 541);
			this.Controls.Add(this.button3);
			this.Controls.Add(this.listBox1);
			this.Controls.Add(this.dataGrid1);
			this.Controls.Add(this.button2);
			this.Controls.Add(this.textBox1);
			this.Name = "Form1";
			this.Text = "Database Viewer";
			((System.ComponentModel.ISupportInitialize)(this.dataGrid1)).EndInit();
			this.ResumeLayout(false);

		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new Form1());
		}



		private void button2_Click(object sender, System.EventArgs e)
		{
			/// Get database file from user:
			openFileDialog1.Filter = @"Access DB (*.mdb)|*.mdb|Excel (*.xls)|*.xls|txt files (*.txt;*.csv)|*.txt;*.csv";
			DialogResult r = openFileDialog1.ShowDialog();
			if(r != DialogResult.OK) return;

			/// Connect to the data source:
			switch(openFileDialog1.FilterIndex)
			{
				case 1:
					// MS Access connection string for OleDB:  (see http://www.connectionstrings.com/)
					oleDbConnection1 = new System.Data.OleDb.OleDbConnection("Provider=Microsoft.Jet.OLEDB.4.0;Data Source=" + openFileDialog1.FileName);
					break;
				case 2:
					// MS Excel connection string for OleDB:  (table names are "sheetname$" for sheets and "regionname" of named regions;  HDR=1st row contains col headers; IMEX=treat intermixed data as strings)
					oleDbConnection1 = new System.Data.OleDb.OleDbConnection("Provider=Microsoft.Jet.OLEDB.4.0;Extended Properties=\"Excel 8.0;HDR=Yes;IMEX=1\";Data Source=" + openFileDialog1.FileName);
					break;
				case 3:
					// Text files connection string for OleDB:  (data source is folder name; tables will be names of textfiles in the folder)
					String f = ((String)openFileDialog1.FileName).Substring(0, ((String)openFileDialog1.FileName).LastIndexOf(@"\"));  // get folder name
					oleDbConnection1 = new System.Data.OleDb.OleDbConnection("Provider=Microsoft.Jet.OLEDB.4.0;Extended Properties=\"text;HDR=Yes;FMT=Delimited\";Data Source=" + f);
					break;
				default:
					return;
			}

			/// Get the database schema, list of table names:  (note ODBC cant do this, only OleDB)
			oleDbConnection1.Open();
			System.Data.DataTable t = oleDbConnection1.GetOleDbSchemaTable(System.Data.OleDb.OleDbSchemaGuid.Tables, new object[]{null,null,null,"TABLE"});

			// put it into the listbox:
			listBox1.DisplayMember = "TABLE_NAME";  // the TABLE_NAME col of t will have the list of table names
			listBox1.DataSource = t;	
			dataGrid1.DataSource = t;
			//oleDbConnection1.Close();
		}

		private void button3_Click(object sender, System.EventArgs e)
		{
			if (oleDbConnection1 == null || oleDbConnection1.State != ConnectionState.Open)  return;
			this.Cursor = Cursors.WaitCursor;

			// Construct the SQL query:   (extract all data from the the table selected in the listbox)
			oleDbCommand1 = new System.Data.OleDb.OleDbCommand("SELECT * FROM [" + listBox1.Text + "]", oleDbConnection1);

			// Create the adapter, with the query:
			// Adapter executes the query to fill the Dataset with data from the DB.
			// With DataSet, all data is captured and stored locally for easy/fast access.
			oleDbDataAdapter1 = new System.Data.OleDb.OleDbDataAdapter(oleDbCommand1);
			dataSet1 = new System.Data.DataSet();
			oleDbDataAdapter1.Fill(dataSet1);

			// Now we can do stuff with the data.
			// Put it in a GUI control:
			dataGrid1.DataSource = dataSet1.Tables[0];

			// Or Process it manually and display it:
			// use 2D indexing Rows[row#][col#] (0-based indexing; can also use column names)
			//MessageBox.Show(dataSet1.Tables[0].Rows[0][5].ToString());
			String s = "";
			foreach(System.Data.DataColumn c in dataSet1.Tables[0].Columns)	// list column names
				s = s + c.ColumnName + "\t";
			s = s + "\r\n";
			foreach(DataRow r in dataSet1.Tables[0].Rows)  // for each row
			{
				foreach(Object v in r.ItemArray)  // for each column
				{
					s = s + v.ToString() + "\t";	// list the value
				}
				s = s + "\r\n";	
			}
			textBox1.Text = s;

			// Instead of Adapter and DataSet, can use a DataReader:
			// DataReader reads only one row from DB at a time, no local cache,
			//  can only loop forward.
//			System.Data.OleDb.OleDbDataReader oleDbDataReader1;
//			oleDbDataReader1 = oleDbCommand1.ExecuteReader();
//			String s1 = "";
//			while(oleDbDataReader1.Read())	// for each row
//			{
//				//oleDbDataReader1.GetValue(0);  // can use column index to get values
//				for(int i=0; i<oleDbDataReader1.FieldCount; i++)	// for each column
//				{
//					s1 = s1 + oleDbDataReader1.GetValue(i).ToString() + "\t";	// list the value
//				}
//				s1 = s1 + "\r\n";
//			}
//			textBox1.Text = s1;

			this.Cursor = Cursors.Default;
		}
	}
}
